﻿// m: Колебания по Бесселю (без реализаций)
// Версия 2.0 от 16.10.2015

window.addEventListener("load", main_BF, false);
function main_BF() {

    // Предварительные установки
    var a0 = 1;                             // масштаб расстояния
    var t0 = 1;                             // масштаб времени
    var m0 = 1;                             // масштаб массы

    var k0 = 2 * Math.PI / t0;              // масштаб частоты
    var C0 = m0 * k0 * k0;                  // масштаб жесткости

    var m = 1 * m0;                 	    // масса
    var C = 0.02 * C0;                 	    // жесткость

    var dx = 1 * a0;                        // шаг сетки по оси x
    var dt = 0.02 * t0;                     // шаг интегрирования по времени

    var N = 100000;                          // количество частиц в каждой реализации
    var T = 15;                             // время расчета

    var koeff = C / (dx * dx) / m * dt;     // коэффициент для уравнения динамики частиц

    // интерфейс программы
    N_number.value = N;
    N_number.oninput = function() {N = parseInt(N_number.value);};
    T_number.value = T;
    T_number.oninput = function() {T = parseInt(T_number.value);};
    button_start.onclick = calculate_new_system;

    checkbox_K.onchange = function() {if (first_calc) draw();};
    checkbox_P.onchange = function() {if (first_calc) draw();};
    checkbox_E.onchange = function() {if (first_calc) draw();};

    var ctx = BF_canvas.getContext("2d");                 // на context происходит рисование
    var w = BF_canvas.width;                              // ширина окна в расчетных координатах
    var h = BF_canvas.height;                             // высота окна в расчетных координатах

    // основная расчетная функция
    var data_K, data_P, data_E;             // данные для графика
    var first_calc = false;                 // произведен ли первый расчет
    var t, n;
    var t_start;
    function calculate_new_system() {
        first_calc = true;
        t = 0;
        n = N + 2;                          // количество узлов по оси x + 2 для ГУ
        span_time_calc.innerHTML = "";
        div_calc_speed.style.display = "none";

        // задание начальных условий
        var P = [];
        for (var i = 1; i < n - 1; i++) {
            P[i] = {};
            P[i].u = 0;
            P[i].v = (2 * Math.random() - 1) * 0.25;
        }
        // периодические граничные условия
        P[0] = P[n - 2];
        P[n - 1] = P[1];

        data_K = []; data_P = []; data_E = [];

        // основной расчет
        t_start = performance.now();
        calculate(P);
    }

    function calculate(P) {
        var t1 = performance.now();

        while (t < T) {
            // расчет энергий для графиков
            var Kin = 0;
            var Pot = 0;
            for (var j = 1; j < n - 1; j++) {Kin += P[j].v * P[j].v;}
            for (var j = 1; j < n - 1; j++) {Pot += Math.pow(P[j].u - P[j - 1].u, 2);}
            Kin = Kin * m;
            Pot = Pot * C;
            data_K.push(Kin);
            data_P.push(Pot);
            data_E.push(Kin + Pot);

            // расчет состояния системы на следующем шаге
            for (var i = 1; i < n - 1; i++) {
                P[i].v += (P[i + 1].u - 2 * P[i].u + P[i - 1].u) * koeff;
            }
            for (var i = 1; i < n - 1; i++) {
                P[i].u += P[i].v * dt;
            }

            t += dt;
            if ((performance.now() - t1) > 200) {
                draw();
                setTimeout( function(){calculate(P); }, 10);
                div_container_progress.style.display = "block";
                button_start.style.display = "none";
                progress_bar.style.width = Math.round(t / T * 100) + "%";
                span_progress.innerHTML = (t / T * 100).toFixed(2) + "%";
                return;
            }
        }
        draw();
        div_container_progress.style.display = "none";
        button_start.style.display = "inline";
        div_calc_speed.style.display = "block";
        span_time_calc.innerHTML = ((performance.now() - t_start) / N * 1000 / Math.ceil(T/t)).toFixed(2);
        return true;
    }

    function draw() {
        ctx.clearRect(0, 0, w, h);              // очистка экрана

        // подпись к оси ("х")
        ctx.font = "20pt Arial";
        ctx.textAlign = "right";
        ctx.textBaseline = "bottom";
        ctx.fillStyle = "#666666";
        ctx.fillText("x", w - 4, h);

        var title_y = 3;                        // начальный отступ для подписи
        var title_y_plus = 30;                  // на столько увеличивается отступ для каждой следующей подписи

        if (checkbox_K.checked) {
            // Кинетическая энергия
            ctx.strokeStyle = "#ff0000";
            ctx.lineWidth = 1;
            ctx.beginPath();
            ctx.moveTo(0, 0);
            for (var i = 1; i < data_K.length; i++) {
                ctx.lineTo(i / (data_K.length - 1) * w, h - data_K[i] / data_K[0] * h);
            }
            ctx.stroke();

            // подпись
            ctx.textAlign = "center";
            ctx.textBaseline = "top";
            ctx.fillStyle = "#ff0000";
            ctx.fillText("K", w - 24, title_y);
            title_y += title_y_plus;
        }

        if (checkbox_P.checked) {
            // Потенциальная энергия
            ctx.strokeStyle = "#0000ff";
            ctx.lineWidth = 1;
            ctx.beginPath();
            ctx.moveTo(0, h - data_P[0] / data_K[0] * h);
            for (var i = 1; i < data_P.length; i++) {
                ctx.lineTo(i / (data_P.length - 1) * w, h - data_P[i] / data_K[0] * h);
            }
            ctx.stroke();

            ctx.textAlign = "center";
            ctx.textBaseline = "top";
            ctx.fillStyle = "#0000ff";
            ctx.fillText("П", w - 24, title_y);
            title_y += title_y_plus;
        }

        if (checkbox_E.checked) {
            // Полная энергия
            ctx.strokeStyle = "#880088";
            ctx.lineWidth = 2;
            ctx.beginPath();
            ctx.moveTo(0, h - data_E[0] / 2 / data_K[0] * h);
            for (var i = 1; i < data_E.length; i++) {
                ctx.lineTo(i / (data_E.length - 1) * w, h - data_E[i] / 2 / data_K[0] * h);
            }
            ctx.stroke();

            ctx.textAlign = "center";
            ctx.textBaseline = "top";
            ctx.fillStyle = "#880088";
            ctx.fillText("E/2", w - 24, title_y);
            title_y += title_y_plus;
        }

    }
}